# SPDX-FileCopyrightText: Emery Hemingway
#
# SPDX-License-Identifier: LicenseRef-Hippocratic-1.1

let
  tests = call:
    {
      fs_report = call ./fs_report.nix { };
      signal = call ./signal.nix { };
    } // call ./solo5 { };

in { self, system, localSystem, crossSystem, genodepkgs, nixpkgs, dhall-haskell, genode-depot, lib }:

let
  hostPkgs = import nixpkgs {
    system = localSystem;
    overlays = [ (self: super: { inherit (dhall-haskell.packages) dhall; }) ];
  };
  depot = builtins.getAttr crossSystem genode-depot.packages;
  testPkgs = genodepkgs;

  linux = tests (import ./driver-linux.nix {
    inherit testPkgs hostPkgs lib depot;
  }).callTest;

  nova = (call: ((tests call) // { pci = call ./pci.nix { }; }))
    (import ./driver-nova.nix {
      inherit system testPkgs hostPkgs lib depot;
    }).callTest;

  testsToList = tests:
    map (test: {
      inherit (test) name;
      value = test;
    }) (builtins.attrValues tests);

in with builtins; listToAttrs (concatLists (map (testsToList) [ linux nova ]))
